package co.enhance.javademoapp;

import android.app.Activity;
import android.os.Bundle;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.PopupWindow;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.util.Log;

import co.enhance.Enhance;
import co.enhance.Enhance.*;
import co.enhance.EnhanceInAppPurchases;
import co.enhance.nativeads.EnhanceAdPositioning;
import co.enhance.nativeads.EnhanceNativeAdRenderer;
import co.enhance.nativeads.EnhanceStaticNativeBannerRenderer;

import java.lang.reflect.Field;
import java.util.Map;

public class MainActivity extends Activity {
	boolean isBannerAdShowing = false;
    String productName = "Enhance_SKU_One";

    private PopupWindow nativeAdsPopupWindow;
    private PopupWindow nativeAdsViewPopupWindow;
    private RelativeLayout nativeAdsRelativeLayout;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		// Set currency callback for offerwalls
		// It is called every time the user receives a reward from offerwall
		Enhance.setReceivedCurrencyCallback(new CurrencyCallback() {
            @Override
            public void onCurrencyGranted(int amount) {
                writeLog("Currency granted: " + amount);
            }
        });

        //Ad Event Callbacks
        Enhance.setOnCompleteCallback(new OnCompleteCallback() {
            @Override
            public void onComplete(String sdkId, String sdkType, String sdkVersion) {
                Log.d("EnhanceCallback", "OnComplete: sdkId=" + sdkId + " sdkType="+sdkType + " sdkVersion=" + sdkVersion);
            }
        });

        Enhance.setOnLoadingCallback(new OnLoadingCallback() {
            @Override
            public void onLoading(String sdkId, String sdkType, String sdkVersion) {
                Log.d("EnhanceCallback", "OnLoading: sdkId=" + sdkId + " sdkType="+sdkType + " sdkVersion=" + sdkVersion);
            }
        });

        Enhance.setOnReadyCallback(new OnReadyCallback() {
            @Override
            public void onReady(String sdkId, String sdkType, String sdkVersion) {
                Log.d("EnhanceCallback", "onReady: sdkId=" + sdkId + " sdkType="+sdkType + " sdkVersion=" + sdkVersion);
            }
        });

        Enhance.setOnUnavailableCallback(new OnUnavailableCallback() {
            @Override
            public void onUnavailable(String sdkId, String sdkType, String sdkVersion) {
                Log.d("EnhanceCallback", "onUnavailable: sdkId=" + sdkId + " sdkType="+sdkType + " sdkVersion=" + sdkVersion);
            }
        });

        Enhance.setOnShowingCallback(new OnShowingCallback() {
            @Override
            public void onShowing(String sdkId, String sdkType, String sdkVersion) {
                Log.d("EnhanceCallback", "onShowing: sdkId=" + sdkId + " sdkType="+sdkType + " sdkVersion=" + sdkVersion);
            }
        });

        Enhance.setOnClickedCallback(new OnClickedCallback() {
            @Override
            public void onClicked(String sdkId, String sdkType, String sdkVersion) {
                Log.d("EnhanceCallback", "onClicked: sdkId=" + sdkId + " sdkType="+sdkType + " sdkVersion=" + sdkVersion);
            }
        });

        Enhance.setOnFailedToShowCallback(new OnFailedToShowCallback() {
            @Override
            public void onFailedToShow(String sdkId, String sdkType, String sdkVersion) {
                Log.d("EnhanceCallback", "onFailedToShow: sdkId=" + sdkId + " sdkType="+sdkType + " sdkVersion=" + sdkVersion);
            }
        });

		// Interstitial Ad
		final Button showInterstitialButton = (Button) findViewById (R.id.showInterButton);
		showInterstitialButton.setOnClickListener(new OnClickListener () {
			@Override
			public void onClick(View v) {
                // Check if ad is ready

                if(!Enhance.isInterstitialReady()) {
                    writeLog("Interstitial ad is not ready");
                    return;
                }

                // Show ad
                Enhance.showInterstitialAd();
                writeLog("Showing interstitial ad");
			}
		});
		
		// Rewarded Ad
		final Button showRewardedAdButton = (Button) findViewById (R.id.showRewardedButton);
		showRewardedAdButton.setOnClickListener(new OnClickListener () {
			@Override
			public void onClick(View v) {
                // Check if ad is ready
                if(!Enhance.isRewardedAdReady()) {
                    writeLog("Rewarded ad is not ready");
                    return;
                }

                // Show ad
                Enhance.showRewardedAd(new RewardCallback() {
                    @Override
                    public void onRewardGranted(int rewardValue, RewardType rewardType) {
                        // Reward is granted, user watched the ad
                        // You can check reward type:

                        if(rewardType == RewardType.ITEM)
                            writeLog("Reward granted (item)");
                        else if(rewardType == RewardType.COINS)
                            writeLog("Reward granted (coins), value: " + rewardValue);
                        else
                            writeLog("Unknown reward granted");
                    }

                    @Override
                    public void onRewardDeclined() {
                        // Reward is declined, user closed the ad before end
                        writeLog("Reward declined");
                    }

                    @Override
                    public void onRewardUnavailable() {
                        // Reward is unavailable (e.g. network error)
                        writeLog("Reward unavailable");
                    }
                });
			}			
		});
		
		// Offerwall
		final Button showOfferwallButton = (Button) findViewById (R.id.showOfferwallButton);
		showOfferwallButton.setOnClickListener(new OnClickListener () {
			@Override
			public void onClick(View v) {
				// Check if offerwall is ready
                if(!Enhance.isOfferwallReady()) {
                    writeLog("Offerwall is not ready");
                    return;
                }

                // Show offerwall
                Enhance.showOfferwall();
                writeLog("Showing offerwall");
			}			
		});

		// Special offer
		final Button showSpecialOfferButton = (Button) findViewById (R.id.showSpecialOfferButton);
		showSpecialOfferButton.setOnClickListener(new OnClickListener () {
			@Override
			public void onClick(View v) {
                // Check if special offer is ready
                if(!Enhance.isSpecialOfferReady()) {
                    writeLog("Special offer is not ready");
                    return;
                }

                // Show special offer
                Enhance.showSpecialOffer();
                writeLog("Showing special offer");
			}			
		});

		// Banner ads
		final Button toggleBannerAdButton = (Button) findViewById (R.id.toggleBannerButton);
		toggleBannerAdButton.setOnClickListener(new OnClickListener () {
			@Override
			public void onClick(View v) {
				// Check if banner ad is already displayed
				if(isBannerAdShowing) {
					isBannerAdShowing = false;
					Enhance.hideBannerAd();
					writeLog("Hiding banner ad");
					return;
				}

				// Banner ad is not shown, check if it's available
				if(!Enhance.isBannerAdReady()) {
					writeLog("Banner ad is not ready");
					return;
				}

				// Show banner ad, it's ready
				isBannerAdShowing = true;
				Enhance.showBannerAdWithPosition(Position.BOTTOM);
				writeLog("Showing banner ad");

                // To show banner ad at the top:
                // FglEnhance.showBannerAdWithPosition(Position.TOP);
			}
		});
		
		// Analytics
		final Button logEventButton = (Button) findViewById (R.id.logEventButton);
		logEventButton.setOnClickListener(new OnClickListener () {
			@Override
			public void onClick(View v) {
				// Send a custom event to analytics network (e.g. Google Analytics)
				Enhance.logEvent("event_type");

				// You can also add an additional parameter to your custom event, for example:
				Enhance.logEvent("event_type", "event_param_key", "event_param_value");

                writeLog("Sent analytics event");
			}			
		});

		// Local Notifications
		final Button enableLocalNotificationButton = (Button) findViewById (R.id.enableLocalNotificationButton);

		enableLocalNotificationButton.setOnClickListener(new OnClickListener () {
			@Override
			public void onClick(View v) {
				// Enable a new local notification
				// It will appear 5 seconds after the app is deactivated
				Enhance.enableLocalNotification("Enhance", "Local Notification!", 5);
				writeLog("Enabled local notification");
			}
		});

		final Button disableLocalNotificationButton = (Button) findViewById(R.id.disableLocalNotificationButton);

		disableLocalNotificationButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				// Cancel previously enabled local notification
				Enhance.disableLocalNotification();
				writeLog("Disabled local notification");
			}
		});

		// In-App Purchases

		final Button purchaseButton = (Button) findViewById(R.id.purchaseButton);

        purchaseButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                // Attempt purchase
                Enhance.purchases.attemptPurchase(productName, new EnhanceInAppPurchases.PurchaseCallback() {
                    @Override
                    public void onPurchaseSuccess() {
                        updateIAPLabel();
                        writeLog("Purchase success");
                    }

                    @Override
                    public void onPurchaseFailed() {
                        writeLog("Purchase failed");
                    }
                });

                writeLog("Attempting purchase");
            }
        });

        final Button consumeButton = (Button) findViewById(R.id.consumeButton);

        consumeButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                // Try to consume
                Enhance.purchases.consume(productName, new EnhanceInAppPurchases.ConsumeCallback() {
                    @Override
                    public void onConsumeSuccess() {
                        updateIAPLabel();
                        writeLog("Consume success");
                    }

                    @Override
                    public void onConsumeFailed() {
                        writeLog("Consume failed");
                    }
                });

                writeLog("Attempting consume");
            }
        });

        final Button showNativeBannerButton = (Button) findViewById(R.id.show_native_popup);

        showNativeBannerButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                // Initialize a new instance of LayoutInflater service
                LayoutInflater inflater = (LayoutInflater) getActivity().getApplicationContext().getSystemService(LAYOUT_INFLATER_SERVICE);

                // Inflate the custom layout/view
                final View customView = inflater.inflate(R.layout.native_ads_popup,null);

                // Initialize a new instance of popup window
                nativeAdsPopupWindow = new PopupWindow(
                        customView,
                        ViewGroup.LayoutParams.WRAP_CONTENT,
                        ViewGroup.LayoutParams.WRAP_CONTENT
                );

                Button closeButton = (Button) customView.findViewById(R.id.back_to_menu);
                closeButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        Enhance.hideNativeBanner();
                        // Dismiss the popup window
                        if(nativeAdsPopupWindow != null && nativeAdsPopupWindow.isShowing())
                            nativeAdsPopupWindow.dismiss();
                    }
                });

                Button showNativeBannerButton = (Button) customView.findViewById(R.id.show_native_banner);
                showNativeBannerButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        writeLog("Showing Native Banner");
                        Enhance.showNativeBanner();
                    }
                });

                Button hideNativeBannerButton = (Button) customView.findViewById(R.id.hide_native_banner);
                hideNativeBannerButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        writeLog("Hiding Native Banner");
                        Enhance.hideNativeBanner();
                        if(nativeAdsViewPopupWindow != null && nativeAdsViewPopupWindow.isShowing())
                            nativeAdsViewPopupWindow.dismiss();
                    }
                });

                Button fillNativeAdapterButton = (Button) customView.findViewById(R.id.show_adapter);
                fillNativeAdapterButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        writeLog("Filling Native Adapter");
                        EnhanceAdPositioning positioning = new EnhanceAdPositioning.Builder()
                                .atIndexes( new int[]{ 1, 3, 5 } )
                                .shift(0)
                                .repeat()
                                .repeatCount(3)
                                .repeatShift(1)
                                .build();

                        ArrayAdapter<String> myAdapter = new ArrayAdapter<String>(getActivity().getApplicationContext(), R.layout.simple_text, R.id.simple_text_view);
                        myAdapter.add("Item 1");
                        myAdapter.add("Item 2");
                        myAdapter.add("Item 3");
                        myAdapter.add("Item 4");
                        myAdapter.add("Item 5");
                        myAdapter.add("Item 6");
                        myAdapter.add("Item 7");
                        myAdapter.add("Item 8");

                        EnhanceNativeAdRenderer adRenderer = new EnhanceStaticNativeBannerRenderer();

                        ListView listView = (ListView) customView.findViewById(R.id.native_ad_list_view);
                        listView.setAdapter(Enhance.createNativeAdAdapter( myAdapter, adRenderer, positioning));
                    }
                });

                Button getNativeViewButton = (Button) customView.findViewById(R.id.show_view);
                getNativeViewButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        writeLog("Getting Native View");
                        if(nativeAdsViewPopupWindow != null && nativeAdsViewPopupWindow.isShowing())
                            return;
                        EnhanceNativeAdRenderer adRenderer = new EnhanceStaticNativeBannerRenderer();
                        View nativeAd = Enhance.getNativeAd(adRenderer);

                        nativeAdsViewPopupWindow = new PopupWindow(
                                nativeAd,
                                ViewGroup.LayoutParams.WRAP_CONTENT,
                                ViewGroup.LayoutParams.WRAP_CONTENT
                        );

                        LinearLayout parent = (LinearLayout) findViewById(R.id.activity_main_id);
                        nativeAdsViewPopupWindow.showAtLocation(parent, Gravity.CENTER,0,0);

                    }
                });

                LinearLayout parent = (LinearLayout) findViewById(R.id.activity_main_id);

                nativeAdsPopupWindow.showAtLocation(parent, Gravity.CENTER,0,0);
            }
        });

        // GDPR

        // Show dialog
        final Button gdprDialogButton = (Button) findViewById (R.id.gdprDialog);
        gdprDialogButton.setOnClickListener(new OnClickListener () {
            @Override
            public void onClick(View v) {
                Enhance.requiresDataConsentOptIn(new OptInRequiredCallback() {
                    // Check if any of the SDKs included require manual user opt-in
                    public void onServiceOptInRequirement(boolean isUserOptInRequired) {
                        if(isUserOptInRequired) {
                            // Display opt-in dialogs for all SDKs that provide them
                            Enhance.showServiceOptInDialogs(new OnDataConsentOptInComplete() {
                                public void onDialogComplete() {
                                    writeLog("Finished displaying opt-in dialogs");
                                }
                            });
                        }
                    }
                });
            }
        });

        // Opt-In
        final Button gdprOptInButton = (Button) findViewById (R.id.gdprOptIn);
        gdprOptInButton.setOnClickListener(new OnClickListener () {
            @Override
            public void onClick(View v) {
                Enhance.serviceTermsOptIn(); // Opt in to all SDKs
                writeLog("Called explicit opt-in [ Enhance.serviceTermsOptIn() ]");
            }
        });

        // Opt-In
        final Button gdprOptOutButton = (Button) findViewById (R.id.gdprOptOut);
        gdprOptOutButton.setOnClickListener(new OnClickListener () {
            @Override
            public void onClick(View v) {
                Enhance.serviceTermsOptOut(); // Opt out of all SDKs
                writeLog("Called explicit opt-out [ Enhance.serviceTermsOptOut() ]");
            }
        });
	}

	private void updateIAPLabel() {
        String title = Enhance.purchases.getDisplayTitle(productName, "Default Title");
        String price = Enhance.purchases.getDisplayPrice(productName, "Default Price");
        int quantity = Enhance.purchases.getOwnedItemCount(productName);

        final TextView label = (TextView) findViewById(R.id.label);
        label.setText(title + " - " + price + " - Qty: " + quantity);
    }

    // Non-Enhance logic
	// Write logs in the TextView
    private void writeLog(CharSequence text) {
		final TextView logsView = (TextView) findViewById (R.id.logsView);
		logsView.setText(text);
		Log.d ("EnhanceDemo", (String) text);
	}

    private static Activity getActivity() {
        try {
            Class activityThreadClass = Class.forName("android.app.ActivityThread");
            Object activityThread = activityThreadClass.getMethod("currentActivityThread").invoke(null);
            Field activitiesField = activityThreadClass.getDeclaredField("mActivities");
            activitiesField.setAccessible(true);
            Map activities = (Map) activitiesField.get(activityThread);
            for(Object activityRecord:activities.values()){
                Class activityRecordClass = activityRecord.getClass();
                Field pausedField = activityRecordClass.getDeclaredField("paused");
                pausedField.setAccessible(true);
                if(!pausedField.getBoolean(activityRecord)) {
                    Field activityField = activityRecordClass.getDeclaredField("activity");
                    activityField.setAccessible(true);
                    Activity activity = (Activity) activityField.get(activityRecord);
                    return activity;
                }
            }
        } catch (Throwable e) {
        }

        return null;
    }
}
